(*
	Brian Bourn, Addisu Petros
*)
open Sast

type str_list ={
mutable	stream_list	: string list;
}

let __list = { stream_list=[]; }

let add_stream _list name =
	_list.stream_list <- name :: _list.stream_list;
	name

let find_in_list name =
	let found = try
		List.find (fun(s) -> s=name) __list.stream_list
	with
		Not_found -> 
			"Not in list"
	in
	found

let empty_list _list =
	_list.stream_list <- []

let string_of_t = function
    Sast.Int -> "int"
  | Sast.Float -> "float"
  | Sast.String -> "string"
  | Sast.Bool -> "bool"
  | _ -> ""

let rec string_of_expr = function
    Sast.Literal(_, l) -> string_of_int l
  | Sast.Id(_, s, e, _) -> s ^(match e with Noexpr -> "" | _ -> " = " ^ 
                        string_of_expr e ^ "")
  | Sast.Binop(_, e1, o, e2) ->
      "(" ^ string_of_expr e1 ^ " " ^
      (match o with
	Ast.Add -> "+" | Ast.Sub -> "-" | Ast.Mult -> "*" | Ast.Div -> "/"
      | Ast.Exp -> "^" | Ast.Equal -> "==" | Ast.Neq -> "!="
      | Ast.Less -> "<" | Ast.Leq -> "<=" | Ast.Greater -> ">" 
      | Ast.Geq -> ">=") 
      ^ " " ^string_of_expr e2 ^")"
  | Sast.String_literal(_, s) ->  "\"" ^ s ^ "\"" 
  | Sast.Float_literal(_, f) ->  string_of_float f 
  | Sast.Bool_literal(_, b) -> string_of_bool b 
  | Sast.Call(_, f, el) ->
      f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  | Sast.Basic_Op(b, s, i, e, j) ->
		(match b with 
        Signal -> (match e with 
        	Sast.Noexpr -> 
				s ^ 
				let str =
					let n = find_in_list s in
						if (n = "Not in list")
						then (
  							ignore (add_stream __list s);
							".value_at(" )
						else
							".peek(" 
				in
				str
				^ (match i with
            Sast.String_(i) -> i
          | Sast.Int_ (i) -> string_of_int i
          ) ^")" 
          	| _ -> "for (int xz=0; xz<1024; xz++) " ^ 
				s ^
				let str =
  					ignore (add_stream __list s);
					".set_value("
				in
				str
			^ string_of_expr e ^ ")" ^ ";"
        )(*".set_value(" ^ (match e with Sast.Basic_Op (_,s,_,_) -> s ^ ".value_at(0)" | _ -> string_of_expr e )^ ")" *)
        |  Array -> s ^ "[" ^ 
          (match i with 
            Sast.String_ (i)-> i 
            | Sast.Int_(i) -> string_of_int i
          ) ^ "]" ^ (match e with Sast.Noexpr -> "" | _ -> "= " ^ string_of_expr e)
      )
  | Sast.Noexpr -> ""


let string_of_var var = match var with
	Primitive(x) -> 
		string_of_t x.a_typ ^ " " ^ x.a_name ^ 
	            (match x.a_exp with Noexpr -> ";\n" | _ -> " = " ^ 
				string_of_expr x.a_exp ^ ";\n")
  | Basic(_) -> ""
  
let string_of_vdecl = function
	Sast.Variable_Dec(_, v) -> string_of_var v


let rec string_of_stmt = function
    Sast.Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Sast.Expr(_, expr) -> 
  		empty_list __list;	
		string_of_expr expr ^ ";\n";
  | Sast.Return(expr) -> "return " ^ string_of_expr expr ^ ";\n";
  | Sast.If(e, s, Block([])) -> "if (" ^ string_of_expr e ^ "){\n" ^ string_of_stmt s^"}\n"
  | Sast.If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ "){\n" ^
      string_of_stmt s1 ^ "\n}else{\n" ^ string_of_stmt s2 ^ "}"
  | Sast.For(e1, e2, e3, s) ->
      "for (" ^ string_of_expr e1  ^ " ; " ^ string_of_expr e2 ^ " ; " ^
      string_of_expr e3  ^ "){\n " ^ string_of_stmt s^ "\n}"
  | Sast.While(e, s) -> "while (" ^ string_of_expr e ^ "){\n " ^ string_of_stmt s^"}"
  | Sast.Print(t, s) -> (*"printf(\" "^ 
                        (match t with 
                                String -> "%s\", \"" ^string_of_expr s ^ "\");"
                              | Int -> "%d\", "^string_of_expr s ^ ");"
                              | Float -> "%f\", "^string_of_expr s ^ ");"
                              (*| Var -> (match s.a_typ with 
                                        String -> "%s\", "^s.a_name ^ ");"
                                      | Int -> "%d\", "^s.a_name ^ ");"
                                      | Float -> "%f\", "^s.a_name ^ ");"
                              ) *)
 
                        )  *)  
  
  			empty_list __list;	
  			(match t with Sast.Float -> "for(int ew=0; ew<1024; ew++) " | _ -> "" ) ^ "cout << " ^  string_of_expr s ^  " <<endl;\n"
  | Sast.Prim_Assign(_, v) -> string_of_var v
  | Sast.Basic_Dec(t, n, l) -> (match t with Signal -> "circular_buffer "^ n ^";\n"
                                | Array -> "int " ^ n ^ "[" ^ string_of_int l ^ "]" ^ ";\n")
  | Sast.Sum(_,var,s, e1, e2, e3) -> empty_list __list; "int " ^ var ^"=0; \n for(int " ^s^"=" ^ string_of_int e1 ^ "; " ^s^"<=" ^ string_of_int e2 ^"; " ^s^"++){\n" ^ var ^"+=" ^ string_of_expr e3 ^ ";\n}"
  | Sast.Basic_Sum(_,var, s,e1,e2, e3) -> empty_list __list;  "for(int ew=0; ew<1024; ew++){\n" ^ "int u=0; \n " ^
     "for(int " ^ s ^ "=" ^ string_of_int e1 ^ "; " ^  s ^ "<" ^ string_of_int e2 ^ "; " ^ s ^ "++){\n " ^ "u+= " ^ string_of_expr e3 ^ ";\n} \n " ^
     var ^ ".set_value(u); \n}"


let string_of_a_fdecl fdecl =
  (match fdecl.a_is_stream with true -> "void " | false ->  "int " (*string_of_t fdecl.a_ret_type*))
  ^fdecl.a_fname ^ "(" ^ String.concat ", " (List.map Ast.string_of_formal fdecl.a_formals) ^ ")\n{\n" ^
  String.concat "" (List.map string_of_stmt fdecl.a_body) ^
  "\n}"

let string_of_prog (vars, funcs) =
    "#include <iostream>\n#include <fstream>\n#include <iomanip>\n#include <string>\n
     #include \"libcirc/circ_buffer.h\"\nusing namespace std;\n"^
  String.concat "" (List.map string_of_vdecl vars) ^ "\n" ^
  String.concat "\n" (List.map string_of_a_fdecl funcs)
