(*
	Abhinav Mishra, Vanshil Shah
*)
type t = 
    Var
  | Func
  | String
  | Int
  | Float
  | Bool  
  | Expression
  | Array	(* type, int *)
  | Signal	(* signal of floats *)
  | No

type i_or_s =
	Int_ of int
  | String_ of string

type a_expr = 
	Literal of t * int
  | String_literal of t * string
  | Bool_literal of t * bool
  | Float_literal of t * float
  | Binop of t * a_expr * Ast.op * a_expr
  | Id of t * string * a_expr * t
  | Call of t * string * a_expr list
  | Basic_Op of t * string * i_or_s * a_expr * t
  | Noexpr

type a_var = {
	a_typ		: t;
	a_name	: string;
	a_exp		: a_expr;
}

type a_stream = {
	str_typ		: t;
	str_name	: string;
}

type a_variable =
	Primitive of a_var
  | Basic of a_stream

type a_stmt =
    Block of a_stmt list
  | Expr of t * a_expr
  | If of a_expr * a_stmt * a_stmt
  | For of a_expr * a_expr * a_expr * a_stmt
  |	Prim_Assign of t * a_variable
  | While of a_expr * a_stmt
  | Print of t * a_expr
  | Return of a_expr
  | Basic_Sum of t * string * string * int * int * a_expr
  | Sum of t * string * string * int * int * a_expr
  | Basic_Dec of t * string * int	(* When creating an Array or a signal, we just need the type, the name, and the size *)

type a_vdecl = 
	Variable_Dec of t * a_variable

type a_fdecl =
{
	a_is_stream	:	bool;
	a_fname		:	string;
	a_ret_type	: 	t option;
	a_formals	:	Ast.formal list;
	a_locals	:	a_variable list;
	a_old_body	:	Ast.stmt list;
	a_body		:	a_stmt	list;
}

type a_program = a_vdecl list  * a_fdecl list
